@@ -10,6 +10,7 @@ from math import ceil, floor
 from typing import Dict, List, Optional
 
 import requests
+from yellowdog_client.common.iso_datetime import iso_timedelta_format
 from yellowdog_client.model import (
     AllNodesInactiveShutdownCondition,
     AllWorkersReleasedShutdownCondition,
@@ -29,16 +30,17 @@ from yd_commands.config import (
     load_config_worker_pool,
 )
 from yd_commands.config_keys import MAINTAIN_INSTANCE_COUNT, USERDATA, USERDATAFILE
-from yd_commands.mustache import (
-    load_json_file_with_mustache_substitutions,
-    load_jsonnet_file_with_mustache_substitutions,
-)
 from yd_commands.printing import (
     print_error,
     print_log,
     print_worker_pool,
     print_yd_object,
 )
+from yd_commands.provision_utils import get_user_data_property
+from yd_commands.variables import (
+    load_json_file_with_variable_substitutions,
+    load_jsonnet_file_with_variable_substitutions,
+)
 from yd_commands.wrapper import CLIENT, CONFIG_COMMON, main_wrapper
 
 
@@ -74,15 +76,15 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
     Directly create the Worker Pool using the YellowDog REST API
     """
 
-    wp_mustache_prefix = "__"
+    wp_variables_prefix = "__"
 
     if wp_json_file.lower().endswith(".jsonnet"):
-        wp_data = load_jsonnet_file_with_mustache_substitutions(
-            wp_json_file, prefix=wp_mustache_prefix
+        wp_data = load_jsonnet_file_with_variable_substitutions(
+            wp_json_file, prefix=wp_variables_prefix
         )
     else:
-        wp_data = load_json_file_with_mustache_substitutions(
-            wp_json_file, prefix=wp_mustache_prefix
+        wp_data = load_json_file_with_variable_substitutions(
+            wp_json_file, prefix=wp_variables_prefix
         )
 
     # Some values are configurable via the TOML configuration file;
@@ -101,7 +103,7 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
             ("requirementNamespace", CONFIG_COMMON.namespace),
             ("requirementTag", CONFIG_COMMON.name_tag),
             ("templateId", CONFIG_WP.template_id),
-            ("userData", get_user_data_property()),
+            ("userData", get_user_data_property(CONFIG_WP)),
             ("imagesId", CONFIG_WP.images_id),
             ("instanceTags", CONFIG_WP.instance_tags),
             ("targetInstanceCount", CONFIG_WP.target_instance_count),
@@ -112,34 +114,37 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
 
         # provisionedProperties insertions
         provisioned_properties = wp_data["provisionedProperties"]
+        autoshutdown_iso: str = iso_timedelta_format(
+            timedelta(minutes=CONFIG_WP.auto_shutdown_delay)
+        )
         for key, value in [
             ("autoShutdown", CONFIG_WP.auto_shutdown),
             (
                 "autoShutdownConditions",
                 [
                     {
-                        "delay": f"PT{CONFIG_WP.auto_shutdown_delay}M",
+                        "delay": autoshutdown_iso,
                         "type": (
                             "co.yellowdog.platform.model."
                             "AllWorkersReleasedShutdownCondition"
                         ),
                     },
                     {
-                        "delay": f"PT{CONFIG_WP.auto_shutdown_delay}M",
+                        "delay": autoshutdown_iso,
                         "type": (
                             "co.yellowdog.platform.model."
                             "AllNodesInactiveShutdownCondition"
                         ),
                     },
                     {
-                        "delay": f"PT{CONFIG_WP.auto_shutdown_delay}M",
+                        "delay": autoshutdown_iso,
                         "type": (
                             "co.yellowdog.platform.model."
                             "UnclaimedAfterStartupShutdownCondition"
                         ),
                     },
                     {
-                        "delay": f"PT{CONFIG_WP.auto_shutdown_delay}M",
+                        "delay": autoshutdown_iso,
                         "type": (
                             "co.yellowdog.platform.model."
                             "NodeActionFailedShutdownCondition"
@@ -148,9 +153,20 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
                 ],
             ),
             ("workerTag", CONFIG_WP.worker_tag),
-            ("nodeBootTimeLimit", f"PT{CONFIG_WP.node_boot_time_limit}M"),
-            ("nodeIdleGracePeriod", f"PT{CONFIG_WP.node_idle_grace_period}M"),
-            ("nodeIdleTimeLimit", f"PT{CONFIG_WP.node_idle_time_limit}M"),
+            (
+                "nodeBootTimeLimit",
+                iso_timedelta_format(timedelta(minutes=CONFIG_WP.node_boot_time_limit)),
+            ),
+            (
+                "nodeIdleGracePeriod",
+                iso_timedelta_format(
+                    timedelta(minutes=CONFIG_WP.node_idle_grace_period)
+                ),
+            ),
+            (
+                "nodeIdleTimeLimit",
+                iso_timedelta_format(timedelta(minutes=CONFIG_WP.node_idle_time_limit)),
+            ),
         ]:
             if provisioned_properties.get(key) is None and value is not None:
                 if key == "autoShutdownConditions":
@@ -278,7 +294,7 @@ def create_worker_pool():
                 requirementName=id,
                 targetInstanceCount=batches[batch_number].initial_nodes,
                 requirementTag=CONFIG_COMMON.name_tag,
-                userData=get_user_data_property(),
+                userData=get_user_data_property(CONFIG_WP),
                 imagesId=CONFIG_WP.images_id,
                 instanceTags=CONFIG_WP.instance_tags,
                 maintainInstanceCount=False,  # Must be false for Worker Pools
@@ -393,21 +409,6 @@ def generate_wp_batch_name(
     return name
 
 
-def get_user_data_property() -> Optional[str]:
-    """
-    Get the 'userData' property, either using the contents of the file
-    specified in 'userDataFile' or using the string specified in 'userData'.
-    Raise exception if both 'userData' and 'userDataFile' are set.
-    """
-    if CONFIG_WP.user_data and CONFIG_WP.user_data_file:
-        raise Exception(f"Only one of '{USERDATA}' or '{USERDATAFILE}' should be set")
-    if CONFIG_WP.user_data:
-        return CONFIG_WP.user_data
-    if CONFIG_WP.user_data_file:
-        with open(CONFIG_WP.user_data_file, "r") as f:
-            return f.read()
-
-
 # Entry point
 if __name__ == "__main__":
     main()
